<?php
namespace Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Services\AprovacaoAlcadasService;

use Controllers\ReportWorkflowController as Report;
use Controllers\SupplyTekApprovalWorkflowController as SupplyTekApproval;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response\Error;

class AprovacaoAlcadasController
{
    private $totalAFAtual = 0;
    private $valorDemanda = 0;
    private $valorAprovacaoAtual;
    private $supplyTekApproval;
    
    public function __construct(AprovacaoAlcadasService $service, Report $report, SupplyTekApproval $supplyTekApproval)
    {
        $this->service = $service;
        $this->report = $report;
        $this->supplyTekApproval = $supplyTekApproval;
    }

    public function getRequestToAprrove(Row $request, Response $response)
    {
        try {
            $row = $request->getRow();
            $idSisDos = $this->service->getIdSisDos();
            $order = $this->service->getApproverOrder($row);
            
            //if (isset($order[0]) && $order[0]["IDEXIGETAPROVE"] == "S" || $idSisDos[0]["IDSISDOS"] == "3") {
            if($idSisDos[0]["IDSISDOS"] == "3"){ 
                $rowToSend = self::arrayWorkflow($row);
                $this->supplyTekApproval->startWorkflow($rowToSend);
            } else {
                if ($row['OPERACAO'] == 'APROVAR') {
                    self::orderApprove($row);
                } else if ($row['OPERACAO'] == 'REPROVAR') {
                    self::orderRepprove($row);
                }
            }
            if ($row['OPERACAO'] == 'APROVAR') {
                $messageToResponse = "Aprovada com sucesso!";
            } else if ($row['OPERACAO'] == 'REPROVAR') {
                $messageToResponse = "Reprovada com sucesso!";
            }
            
            $messageToResponse = new Response\Message($messageToResponse);
            $response->addMessage($messageToResponse);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function orderApprove($approvalData)
    {
        $orderTag = 'APROVADO';
        $orderTagFinalize = 'APROVADA';
        $justifyMessage = $approvalData['JUSTIFICATIVA'] == '' ? 'Aprovado.' : $approvalData['JUSTIFICATIVA'];
        $orderData = $this->service->getOrder($approvalData);
        
        //$usaPorcentagem  = ($orderData['IDTIPOVRAPROV'] == 'PORCENTAGEM');
        $usaPorcentagem = false;
        
        $maxValue = $this->service->getApproversMaxValue($orderData)['VALORMAXIMO'];
        $paramValorUtilizado = self::getParamConsisteValorAprovacao($orderData);
        $totalOrderValue = self::getTotal($orderData, $paramValorUtilizado, $approvalData, $usaPorcentagem);
        
        if ($totalOrderValue > $maxValue && $usaPorcentagem == false) {
            if ($paramValorUtilizado == 'M') {
                throw new \Exception('Não existe gestor com valor de aprovação maior que o valor total da AF no período.');
            }
            throw new \Exception('Não existe gestor com valor de aprovação maior que o valor total da AF.');
        }

        // DEBUG - Comentar a linha abaixo para que não seja realizado o update.
        self::updateApprovalWorkflow($approvalData, $orderData, $justifyMessage, $orderTag);
        $subjectParam = self::setAprroveData($orderData, $justifyMessage, $orderTagFinalize, $approvalData, $totalOrderValue);
        $this->report->sendApprovalReport($orderData, $subjectParam, $justifyMessage, $approvalData['CDOPERLOG']);
    }

    public function orderRepprove($approvalData)
    {
        $orderTagUpdate = 'RECUSADO';
        $orderTagFinalize = 'RECUSADA';
        $subjectParam = 'REPROVACAO_ALCADA';
        $justifyMessage = $approvalData['JUSTIFICATIVA'];
        $orderData = $this->service->getOrder($approvalData);
        if (!empty($justifyMessage)) {
            self::updateApprovalWorkflow($approvalData, $orderData, $justifyMessage, $orderTagUpdate);
            self::finalizeWorkflow($orderData, $orderTagFinalize);
            $this->report->sendRepproveReport($orderData, $subjectParam, $justifyMessage, $approvalData['CDOPERLOG']);
        }
    }

    private function updateApprovalWorkflow($approvalData, $orderData, $justifyMessage, $orderTag)
    {
        $userLog = $approvalData['CDOPERLOG'];
        $approveCode = $orderData['NRAPROVALCADA'];
        $this->service->updateWorkflow($approveCode, $orderTag, $justifyMessage, $userLog);
    }

    private function finalizeWorkflow($orderData, $orderTag)
    {
        $nrAutoForn = $orderData['NRAUTOFORN'];
        $cdSupplier = $orderData['CDFORNECED'];
        $this->service->updateOrder($orderData, $orderTag);
        $this->service->executeCONSISTE_ORCAMENTO($orderData['CDFILDEST'], $orderData['CDFILLANC'], $orderData['NRAUTOFORN']);
    }

    private function setApprovalOnWorkflow($orderData, $userCode, $justifyMessage)
    {
        $nrApprove = $orderData['NRAPROVALCADA'];
        $this->service->setApprover($userCode, $nrApprove);
    }

    private function setAprroveData($orderData, $justifyMessage, $orderTag, $approvalData, $totalOrderValue)
    {
        $approver = $this->service->getApprover($orderData);
        
        if (empty($approver)) {
            self::finalizeWorkflow($orderData, $orderTag);
            
            return 'LIBERACAO_DEMANDA';
        }
        
        /*
        // DEBUG
        $excedeu = $this->isValorExcedente() ? 'Sim' : 'Não';
        echo("========================================\n");
        echo(">> function setAprroveData << \n");
        echo('Valor total excedeu? ');
        var_dump($excedeu);
        die;
        */
        
        if ($this->isValorExcedente()) {
            $userCode = $approver[0]['CDOPERADOR'];
            self::setApprovalOnWorkflow($orderData, $userCode, $justifyMessage);
            
            return 'APROVACAO_ALCADA';
        }
        
        self::finalizeWorkflow($orderData, $orderTag);
        
        return 'APROVACAO_AUTOMATICA';
    }
    
    private function convertOrderValue($value)
    {
        $value = str_replace("R$", "", $value);
        
        return (int)$value;
    }
    
    /**
     * Verifica se o total disponível ultrapassa o permitido, sendo necessário
     * ir para o próximo aprovador.
     * 
     * Se o valor for positivo, significa que o valor ultrapassou o total disponível e,
     * portanto, deve ir para o próximo aprovador.
     * Do contrário, o valor está dentro do permitido e é aprovado automaticamente.
     */
    private function isValorExcedente()
    {
        $totalDisponivel = $this->getTotalDisponivel();
        return $totalDisponivel > 0;
    }
    
    public function transformValueMaskToFloat($value)
    {
        $arrayReplace = array(
            'R$',
            '.',
        );
        
        $newValue = str_replace(',', '.', str_replace($arrayReplace, '', $value));
        
        return (float) $newValue;
    }

    public function getTotal($dadosDemanda, $paramValorUtilizado, $approvalData, $usaPorcentagem = false)
    {
        if ($paramValorUtilizado == 'M') {
            $dataEntregaAF    = $this->getDataEntregaAF($dadosDemanda);
            $periodoOrcamento = $this->getPeriodoOrcamento($dadosDemanda, $dataEntregaAF);
            $aprovadorAtual = $this->service->findAprovadorAtual($approvalData)[0];
            if($usaPorcentagem){
                //$aprovadorAtual['VRAPROVCMP'] = self::buscaValorTotalPorcentagem($dadosDemanda, $aprovadorAtual);
            }
            
            $this->valorAprovacaoAtual = $aprovadorAtual['VRAPROVCMP'];
            $this->valorDemanda = $this->getValorDemanda($dadosDemanda, $dataEntregaAF, $periodoOrcamento);
            $this->totalAFAtual = $dadosDemanda['VRTOTALAF'];
            
            $totalGasto = $this->getTotalGastoNoPeriodo();
        } else {
            $totalGasto = self::getTotalDia($dadosDemanda) + $dadosDemanda['VRTOTALAF'];
        }
        
        return $totalGasto;
    }

    /**
     * Retorna o total disponível, calculado entre Total Gasto menos o Total Disponível para Uso.
     * 
     * Os valores que compõe os valores utilizados no cálculo são recuperados
     * na função getTotalGastoNoPeriodo.
     */
    private function getTotalDisponivel()
    {
        return $this->getTotalGastoNoPeriodo() - $this->getTotalDisponivelParaUso();
    }
    
    /**
     * O total gasto é a soma dos seguintes valores:
     * - resultado da função VALOR_DEMANDA (detalhes do funcionamento no service);
     * - valor total da AF atual.
     */
    private function getTotalGastoNoPeriodo()
    {
        return $this->valorDemanda + $this->totalAFAtual;
    }
    
    /**
     * Total disponível para uso é o valor de aprovação do aprovador atual.
     * O valor do orçamento disponível já foi descontado na VALOR_DEMANDA.
     */
    private function getTotalDisponivelParaUso()
    {
        return $this->valorAprovacaoAtual;
    }
    
    public function getValorDemanda($dadosDemanda, $dataEntregaAF, $periodoOrcamento)
    {
        $paramsValorDemanda = [
            'DTPROGENTAF' => $dataEntregaAF,
            'CDFILDEST'   => $dadosDemanda['CDFILDEST'],
            'NRDEMANDA'   => $dadosDemanda['NRAUTOFORN'],
            'IDPERIODOORCA'   => $periodoOrcamento['V_IDPERIODOORCA'],
            'NRPERIODOORCA'   => $periodoOrcamento['V_NRPERIODOORCAGOP'],
        ];
        
        return $this->service->getValorDemanda($paramsValorDemanda)[0]['VRDEMANDA'];
    }
    
    public function getTotalDia($dadosDemanda)
    {
        $params = [
            "CDFILDEST"    => $dadosDemanda['CDFILDEST'],
            "CDFILLANC"    => $dadosDemanda['CDFILLANC'],
            "NRDEMANDA"    => $dadosDemanda['NRAUTOFORN'],
        ];
        
        return $this->service->totalDiaAF($params)[0]['TOTAL'];
    }

    public function getDiaInicioOrcamento($dadosDemanda)
    {
        return $this->service->findDiaInicioOrcamento($dadosDemanda['CDFILDEST'])[0]['DIAINIORCA'];
    }
    
    public function getPeriodoOrcamento($dadosDemanda, $dataEntregaAF)
    {
        $diaInicioOrcamento = $this->getDiaInicioOrcamento($dadosDemanda);
        
        $dataOrcamento = $this->getDataOrcamento($diaInicioOrcamento, $dataEntregaAF);
        
        $paramsPeriodoOrcamento = [
            'V_DATAINI' => $dataOrcamento['V_DATAINI'],
            'DIAINIORCA' => $diaInicioOrcamento,
        ];
        
        return $this->service->findPeriodoOrcamento($paramsPeriodoOrcamento)[0];
    }
    
    public function getDataOrcamento($diaInicioOrcamento, $dataEntregaAF)
    {
        $params = [
            'DIA' => $diaInicioOrcamento,
            'DATA' => $dataEntregaAF,
        ];
        
        return $this->service->findDataOrcamento($params)[0];
    }
    
    public function getDataEntregaAF($dadosDemanda)
    {
        $params = [
            'CDFILDEST' => $dadosDemanda['CDFILDEST'],
            'CDFILLANC' => $dadosDemanda['CDFILLANC'],
            'NRDEMANDA' => $dadosDemanda['NRAUTOFORN'],
        ];
        
        return $this->service->findDataEntregaAF($params)[0]['DTPROGENTAF'];
    }
    
    public function getParamConsisteValorAprovacao($dadosDemanda)
    {
        $params = [
            'CDFILDEST'      => $dadosDemanda['CDFILDEST'],
            'NRCONFIGALCADA' => $dadosDemanda['NRCONFIGALCADA'],
        ];
        
        return $this->service->findFilialAlcada($params)[0]['IDCONSISTVRAPROV'];
    }
    
    //////////////////////////////////////////////////////////////////////////////////////////////////
    ///                              FUNÇÔES EM DESENVOLVIMENTO                                   ////
    //////////////////////////////////////////////////////////////////////////////////////////////////
    public function arrayWorkflow($row)
    {
        $vrAprovCmp = $this->convertOrderValue($row["VRAPROVACAO"]);
        $row["JUSTIFICATIVA"] = $row['JUSTIFICATIVA'] == '' ? 'Aprovado.' : $row['JUSTIFICATIVA'];
        
        $rowToSend = array(
            "CDFILDEST"         => $row["CDFILDEST"],
            "CDFILLANC"         => $row["CDFILLANC"],
            "NRAUTOFORN"        => $row["NRAUTOFORNE"],
            "CDOPERADOR"        => $row["CDOPERLOG"],
            "STATUS"            => $row["OPERACAO"][0],
            "DSJUSTIFIC"        => $row["JUSTIFICATIVA"],
            "NRAPROVALCADA"     => $row["NRAPROVALCADA"],
            "NRCONFIGALCADA"    => $row["NRCONFIGALCADA"],
            "VRAPROVCMP"        => $vrAprovCmp
        );
        
        return $rowToSend;
    }
    
    public function buscaValorTotalPorcentagem($dadosDemanda, $aprovadorAtual){
        $orcamento = self::buscaOrcamento($dadosDemanda, $aprovadorAtual);
        $vraprovcmp = $orcamento[0] * $aprovadorAtual['VRAPROVCMP'] / 100;
        
        return $vraprovcmp;
    }
    
    public function buscaOrcamento ($dadosDemanda, $aprovadorAtual){
        $C_FILIAL = $this->service->buscaC_Filial();
        $C_AUTOFORNE = $this->service->buscaC_AUTOFORNE();
        $V_DATAINI = $this->service->buscaV_DATA($C_FILIAL['DIAINIORCA'], $C_AUTOFORNE['DTPROGENTAF']);
        $V_DATAFIM =  $this->service->createDATAFIM($V_DATAINI);
        $periodo = $this->service->buscaPeriodo($dadosDemanda,$C_FILIAL['DIAINIORCA'],$V_DATAINI);
    }
    
    //////////////////////////////////////////////////////////////////////////////////////////////////
    ///                                     FUNÇÔES DA TELA                                       ////
    //////////////////////////////////////////////////////////////////////////////////////////////////
    public function getAlcadasToAprroval(Request\Filter $request, Response $response)
    {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $conditions     = self::getConditionsFromFilter($filterCriteria);
            
            $conditions['P_DTINCLUSAO_INI'] = $conditions['DTINCLUSAO']['start'];
            $conditions['P_DTINCLUSAO_FIN'] = $conditions['DTINCLUSAO']['end'];
            
            if (empty($conditions['DTINCLUSAO']['start']) && empty($conditions['DTINCLUSAO']['end'])) {
                $conditions['P_TDTINCLUSAO'] = 'T';
            } else {
                $conditions['P_TDTINCLUSAO'] = '';
            }
            
            $conditions['P_DTENTREGAAF_INI'] = $conditions['DTENTREGAAF']['start'];
            $conditions['P_DTENTREGAAF_FIN'] = $conditions['DTENTREGAAF']['end'];
            
            if (empty($conditions['DTENTREGAAF']['start']) && empty($conditions['DTENTREGAAF']['end'])) {
                $conditions['P_TDTENTREGAAF'] = 'T';
            } else {
                $conditions['P_TDTENTREGAAF'] = '';
            }
            
            $alcadas        = $this->service->getAlcadas($conditions);
            $dataset        = self::formatAlcadasToResponse($alcadas);
            
            $response->addDataSet(new DataSet('alcadas', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getConditionsFromFilter($filter)
    {
        $conditions = array();
        
        foreach ($filter->getConditions() as $filter) {
            if (is_array($filter['value']) && strpos($filter['columnName'], 'DT') != false) 
                $conditions[$filter['columnName']] = implode("_", $filter['value']) . '_';
            else 
                $conditions[$filter['columnName']] = $filter['value'];
        }
        
        return $conditions;
    }
    
    public function formatAlcadasToResponse($alcadas)
    {
        $formattedAlcada = array();
        
        foreach ($alcadas as $key=>$alcada) {
            $column = $this->service->defineColumn($alcada['IDAPROVADO']);
            $label = $alcada['NRDEMANDA'].' - '.$alcada['NMFILDEST'];
            $alcada = self::concatFilialData($alcada);
            $alcada = self::applyMaskOnFields($alcada);
            $description =  'AF: '.$alcada['NRDEMANDA'].
                            '<br>Entrega: '.explode(' ', $alcada['DTPROGENTAF'])[0].
                            '<br>Fornecedor: '.$alcada['NMFANTFORN'].
                            '<br>Prioridade: '.$alcada['IDCLASSIFIC'].
                            '<br>Total AF: '.$alcada['VRTOTALAF'].
                            '<br>Alçada: '.$alcada['NMALCADA'].
                            '<br>';
                            
            $formattedAlcada[$key] = self::arrayMount($column, $label, $description, $alcada);
        }
        
        return $formattedAlcada;
    }
    
    public function arrayMount($column, $label, $description, $alcada)
    {
        $alcada["column"]      = $column;
        $alcada["label"]       = $label;
        $alcada["movable"]     = false;
        $alcada["description"] = $description;
        
        return $alcada;
    }
    
    public function concatFilialData($dataset)
    {
        $dataset['CDNMFILDEST'] = $dataset['CDFILDEST'].' - '.$dataset['NMFILDEST'];
        
        return $dataset;
    }
    
    public function applyMaskOnFields($alcada)
    {
        $alcada['IDCLASSIFIC'] = $this->service->getNameClassific($alcada['IDCLASSIFIC']);
        $alcada['IDAPROVADO']  = $this->service->getNameIdAprovado($alcada['IDAPROVADO']);
        $alcada['VRAPROVCMP']  = self::currencyMaskApply($alcada['VRAPROVCMP']);
        $alcada['VRTOTALAF']   = self::currencyMaskApply($alcada['VRTOTALAF']);
        
        return $alcada;
    }
    
    public function currencyMaskApply($value)
    {
        return 'R$' . number_format($value, 2, ',', '.');
    }
}
