<?php
namespace Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Util\WkfUtilities;

class CadastroUsuarioSupplyTekService {
    public function __construct(WkfUtilities $wkfUtilities) {
        $this->wkfUtilities = $wkfUtilities;
    }
    
    public function insertUserSupplyTek($row) {
        try {
            $query = "INSERT INTO ACESSOSUPPLY
                        (CDFILIAL, CDOPERADOR, IDPERFILACES, IDATIVO, NRORG)
                      VALUES
                        (:CDFILIAL, :CDOPERADOR, :IDPERFILACES, :IDATIVO, :NRORG)";
                        
            foreach($row['CDFILIAL'] as $filial) {
                
                $params = [
                    'CDFILIAL'          => $filial['CDFILIAL'],
                    'CDOPERADOR'        => $row['CDOPERADOR'],
                    'IDPERFILACES'      => $row['IDPERFILACES'],
                    'IDATIVO'           => $row['IDATIVO'],
                    'NRORG'             => $row['NRORG']
                ];
                
                $this->wkfUtilities->crud($query, $params);
            }
        }
        catch (Exception $e) {
             throw $e;
        }
    }
    
    public function saveUnidadePadrao($row) {
        try {
            $query = "UPDATE ACESSOSUPPLY 
                        SET IDFILIALPADRAO = :P_IDFILIALPADRAO
                    WHERE CDOPERADOR = :P_CDOPERADOR AND CDFILIAL = :P_CDFILIAL";
                        
                $params = [
                    'P_CDFILIAL'          => $row['CDFILIALPADRAO'],
                    'P_CDOPERADOR'        => $row['CDOPERADOR'],
                    'P_IDFILIALPADRAO'    => 'S'
                ];
                $this->wkfUtilities->crud($query, $params);
            
        }
        catch (Exception $e) {
             throw $e;
        }
    }
    
    public function updateFilialNaoPadrao($row) {
        try {
            $query = "UPDATE ACESSOSUPPLY 
                        SET IDFILIALPADRAO = :P_IDFILIALPADRAO
                    WHERE CDOPERADOR = :P_CDOPERADOR";
                        
                $params = [
                    'P_CDOPERADOR'        => $row['CDOPERADOR'],
                    'P_IDFILIALPADRAO'    => 'N'
                ];
                $this->wkfUtilities->crud($query, $params);
            
        }
        catch (Exception $e) {
             throw $e;
        }
    }
    
    public function deleteUserSupplyTek($row) {
        try {
            $query = "DELETE FROM ACESSOSUPPLY 
                       WHERE CDOPERADOR      = :P_CDOPERADOR
                         AND CDFILIAL        = :P_CDFILIAL
                         AND IDPERFILACES = :P_IDPERFILACES
                         AND NRORG = :P_NRORG";
            
                $params = [
                    'P_CDFILIAL'        => $row['CDFILIAL'],
                    'P_CDOPERADOR'      => $row['CDOPERADOR'],
                    'P_IDPERFILACES'    => $row['IDPERFILACES'],
                    'P_NRORG'           => $row['NRORG']
                ];
                $this->wkfUtilities->crud($query, $params);    
        }
        catch (Exception $e) {
            throw $e;
        }
    }
    public function selectOperator($row) {
        try {
           
            $query = "SELECT * FROM ACESSOSUPPLY 
                        WHERE CDOPERADOR = :P_CDOPERADOR";
                        
                $params = [
                    'P_CDOPERADOR' => $row['CDOPERADOR']
                ];
                return $this->wkfUtilities->crud($query, $params);
            
        }
        catch (Exception $e) {
            throw $e;
        }
    }
    
    public function verifyOperator($cdOperador)
    {
        try {
            $sql = "SELECT DISTINCT
                        IDPERFILACES
                    FROM
                        ACESSOSUPPLY
                    WHERE
                        CDOPERADOR = :P_CDOPERADOR";
                        
            $params = [
                'P_CDOPERADOR' => $cdOperador
            ];
            
            $result = $this->wkfUtilities->crud($sql, $params);
            
            return $result;
        }
        catch (Exception $e) {
            throw $e;
        }
    }
    
    public function verifyUnitFromOperator($cdOperador, $cdFilial)
    {
        try {
            $sql = "SELECT DISTINCT
                        IDPERFILACES
                    FROM
                        ACESSOSUPPLY
                    WHERE
                        CDOPERADOR = :P_CDOPERADOR
                        AND CDFILIAL = :P_CDFILIAL";
                        
            $params = [
                'P_CDOPERADOR' => $cdOperador,
                'P_CDFILIAL' => $cdFilial
            ];
            
            $result = $this->wkfUtilities->crud($sql, $params);
            
            return $result;
        }
        catch (Exception $e) {
            throw $e;
        }
    }
    
    public function addOperator($row)
    {
        try {
            $query = "  INSERT INTO ACESSOSUPPLY
                        (
                            CDFILIAL, 
                            CDOPERADOR, 
                            IDPERFILACES, 
                            IDATIVO, 
                            NRORG,
                            IDFILIALPADRAO
                        )
                        VALUES
                        (
                            :CDFILIAL, 
                            :CDOPERADOR, 
                            :IDPERFILACES, 
                            :IDATIVO, 
                            :NRORG,
                            :IDFILIALPADRAO
                        )";

            $params = [
                'CDFILIAL' => $row['CDFILIAL'],
                'CDOPERADOR' => $row['CDOPERADOR'],
                'IDPERFILACES' => $row['IDPERFILACES'],
                'IDATIVO' => $row['IDATIVO'],
                'NRORG' => $row['NRORG'],
                'IDFILIALPADRAO' => $row['IDFILIALPADRAO']
            ];

            $this->wkfUtilities->crud($query, $params);
        }
        catch (Exception $e) {
             throw $e;
        }
    }
}

